@@ -47,6 +47,7 @@ static constexpr const char* DEFAULT_SYNTAX_JS =
     "re2c:YYFILL:naked = 0;\n"
     "re2c:YYFN = [\"<undefined-function-name>;<undefined-function-type>\"];\n"
     "re2c:yyfn:sep = \";\";\n"
+    "re2c:yyfn:throw = \"\";\n"
     "re2c:yycond = \"yycond\";\n"
     "re2c:yyctable = \"yyctable\";\n"
     "re2c:yyaccept = \"yyaccept\";\n"
@@ -87,6 +88,7 @@ static constexpr const char* DEFAULT_SYNTAX_JS =
     "re2c:debug-output = 0;\n"
     "re2c:computed-gotos = 0;\n"
     "re2c:computed-gotos:threshold = 9;\n"
+    "re2c:computed-gotos:relative = 0;\n"
     "re2c:nested-ifs = 0;\n"
     "re2c:case-insensitive = 0;\n"
     "re2c:case-inverted = 0;\n"
@@ -159,6 +161,7 @@ static constexpr const char* DEFAULT_SYNTAX_JS =
     "code:const_local = topindent \"const \" name \" = \" init nl;\n"
     "code:const_global = code:const_local;\n"
     "\n"
+    "// Not sure if there's a way to declare array with immutable elements in JS.\n"
     "code:array_local =\n"
     "    topindent \"const \" name \" = [\" nl indent\n"
     "        [row: topindent [elem{0:-2}: elem \", \"] [elem{-1}: elem \",\"] nl]\n"
@@ -178,6 +181,7 @@ static constexpr const char* DEFAULT_SYNTAX_JS =
     "code:type_uint = <undefined>;\n"
     "code:type_yybm = <undefined>;\n"
     "code:type_yytarget = <undefined>;\n"
+    "code:type_yyctable = <undefined>;\n"
     "\n"
     "code:cmp_eq = \"===\";\n"
     "code:cmp_ne = \"!==\";\n"
@@ -221,6 +225,8 @@ static constexpr const char* DEFAULT_SYNTAX_JS =
     "code:continue = topindent \"continue\" (.loop_label ? \" \" label) nl;\n"
     "\n"
     "code:goto = <undefined>;\n"
+    "code:cgoto = <undefined>;\n"
+    "code:cgoto_data = <undefined>;\n"
     "\n"
     "code:fndecl = <undefined>;\n"
     "code:fndef = <undefined>;\n"
@@ -381,4 +387,6 @@ static constexpr const char* DEFAULT_SYNTAX_JS =
     "code:yybm_filter = yych \" & ~0xFF\";\n"
     "\n"
     "code:yybm_match = \"(\" yybm \"[\" offset \"+\" yych \"] & \" mask \") !== 0\";\n"
+    "\n"
+    "code:yytarget_filter = <undefined>;\n"
     ;
