@@ -1454,11 +1454,7 @@ LOCAL_NODISCARD(Ret expand_tags_directive(Output& output, Code* code)) {
     return Ret::OK;
 }
 
-static void gen_cond_enum(
-        Scratchbuf& buf,
-        OutAllocator& alc,
-        Code* code,
-        const opt_t* opts,
+static void gen_cond_enum(Scratchbuf& buf, OutAllocator& alc, Code* code, const opt_t* opts,
         const StartConds& conds) {
     DCHECK(opts->target == Target::CODE);
 
@@ -1468,19 +1464,16 @@ static void gen_cond_enum(
     if (code->fmt.format) {
         const char* fmt = code->fmt.format;
         const char* sep = code->fmt.separator;
-        uint32_t cond_number = 0;
         for (const StartCond& cond : conds) {
             if (sep != nullptr && &cond != first_cond) {
                 buf.cstr(sep);
             }
             std::ostringstream s(fmt);
-            // The main substitution (the one allowing unnamed sigil) must go last, or else it will
-            // erroneously substitute all the named ones.
-            size_t cid = opts->code_model == CodeModel::GOTO_LABEL ? cond_number : cond.number;
-            argsubst(s, opts->api_sigil, "num", false, cid);
+            // The main substitution (the one allowing unnamed sigil) must go last,
+            // or else it will erroneously substitute all the named ones.
+            argsubst(s, opts->api_sigil, "num", false, cond.number);
             argsubst(s, opts->api_sigil, "cond", true, cond.name);
             buf.str(s.str());
-            ++cond_number;
         }
         buf.cstr("\n");
 
@@ -1491,46 +1484,53 @@ static void gen_cond_enum(
         // prepare an array of enum member names
         const char** ids = alc.alloct<const char*>(conds.size()), **i = ids;
         for (const StartCond& cond : conds) *i++ = buf.str(cond.name).flush();
-        // prepare an array of enum member numbers (only needed in loop/switch or rec/func mode)
-        uint32_t* nums = nullptr;
-        if (opts->code_model != CodeModel::GOTO_LABEL) {
-            uint32_t* j = nums = alc.alloct<uint32_t>(conds.size());
-            for (const StartCond& cond : conds) *j++ = cond.number;
-        }
+        // prepare an array of enum member numbers
+        uint32_t* nums = alc.alloct<uint32_t>(conds.size()), *j = nums;
+        for (const StartCond& cond : conds) *j++ = cond.number;
         // construct enum code item in place of the old code item
         init_code_enum(code, opts->api_cond_type.c_str(), conds.size(), ids, nums);
     }
 }
 
 LOCAL_NODISCARD(Ret add_condition_from_block(
-        const OutputBlock& block, StartConds& conds, StartCond cond)) {
+        const OutputBlock& block, const opt_t* opts, StartConds& conds, StartCond cond)) {
     // Condition prefix is specific to the block that defines it. If a few blocks define conditions
     // with the same name, but a different prefix, they should have different enum entries.
     cond.name = block.opts->cond_enum_prefix + cond.name;
 
-    for (const StartCond& c : conds) {
-        if (c.name == cond.name) {
-            if (c.number == cond.number) {
-                // A duplicate condition, it's not an error but don't add it.
-                return Ret::OK;
-            } else {
-                // An error: conditions with idetical names but different numbers.
-                RET_FAIL(error("cannot generate condition enumeration: condition `%s` has "
-                               "different numbers in different blocks (use `re2c:condenumprefix` "
-                               "configuration to set per-block prefix)",
-                               cond.name.c_str()));
-            }
-        }
+    // In loop/switch or rec/func mode condition numbers are the numeric indices of their start
+    // DFA state. In goto/label mode conditions are renumbered: they are assigned consecutive
+    // numbers starting from zero. Conditions from different blocks are merged together in this
+    // enum, and the enumeration is not per-block but global across all contributing blocks.
+    bool renumber = opts->code_model == CodeModel::GOTO_LABEL;
+
+    auto same = std::find_if(conds.begin(), conds.end(),
+            [=](const StartCond& c){ return c.name == cond.name; });
+
+    if (same == conds.end()) {
+        // Ok, a new condition, add it.
+        if (renumber) cond.number = static_cast<uint32_t>(conds.size());
+        conds.push_back(cond);
+    } else if (same->number == cond.number || renumber) {
+        // Condition with the same name is already on the list: it's fine if the numbers agree,
+        // or if the numbers are reassigned (in goto/label mode). We don't care about `defined`
+        // flags here (they matter for condition dispatch, which is generated on per-block basis).
+    } else {
+        // Error: different conditions with identical names.
+        RET_FAIL(error(
+            "cannot generate condition enumeration: condition `%s` has different numbers in "
+            "different blocks (use `re2c:condenumprefix` configuration to set per-block prefix)",
+            cond.name.c_str()));
     }
 
-    conds.push_back(cond);
     return Ret::OK;
 }
 
-LOCAL_NODISCARD(Ret add_conditions_from_blocks(const blocks_t& blocks, StartConds& conds)) {
+LOCAL_NODISCARD(Ret add_conditions_from_blocks(
+        const blocks_t& blocks, const opt_t* opts, StartConds& conds)) {
     for (const OutputBlock* block : blocks) {
         for (const StartCond& cond : block->conds) {
-            CHECK_RET(add_condition_from_block(*block, conds, cond));
+            CHECK_RET(add_condition_from_block(*block, opts, conds, cond));
         }
     }
     return Ret::OK;
@@ -1552,12 +1552,12 @@ LOCAL_NODISCARD(Ret expand_cond_enum(Output& output, Code* code)) {
     StartConds conds;
     if (code->fmt.block_names == nullptr) {
         // Gather conditions from all blocks in the output and header files.
-        CHECK_RET(add_conditions_from_blocks(output.cblocks, conds));
-        CHECK_RET(add_conditions_from_blocks(output.hblocks, conds));
+        CHECK_RET(add_conditions_from_blocks(output.cblocks, globopts, conds));
+        CHECK_RET(add_conditions_from_blocks(output.hblocks, globopts, conds));
     } else {
         // Gather conditions from the blocks on the list.
         CHECK_RET(find_blocks(output, code->fmt.block_names, output.tmpblocks, "conditions"));
-        CHECK_RET(add_conditions_from_blocks(output.tmpblocks, conds));
+        CHECK_RET(add_conditions_from_blocks(output.tmpblocks, globopts, conds));
     }
 
     // Do not generate empty condition enum. Some compilers or language standards allow it, but
@@ -1599,8 +1599,8 @@ static CodeList* gen_cond_goto_binary(
 
     CodeList* stmts = code_list(alc);
     if (lower == upper) {
-        buf.str(opts->cond_label_prefix).str(block.conds[lower].name);
-        append(stmts, code_goto(alc, buf.flush()));
+        const char* label = buf.str(opts->cond_label_prefix).str(block.conds[lower].name).flush();
+        append(stmts, code_goto(alc, label));
     } else {
         const size_t middle = lower + (upper - lower + 1) / 2;
         CodeList* if_then = gen_cond_goto_binary(output, getcond, lower, middle - 1);
@@ -1637,13 +1637,10 @@ static CodeList* gen_cond_goto(Output& output) {
         append(stmts, gen_cond_goto_binary(output, getcond, 0, ncond - 1));
     } else {
         warn_cond_ord = false;
-
         CodeCases* ccases = code_cases(alc);
         for (const StartCond& cond : conds) {
             CodeList* body = code_list(alc);
-            buf.str(opts->cond_label_prefix).str(cond.name);
-            append(body, code_goto(alc, buf.flush()));
-
+            append(body, code_goto(alc, buf.str(opts->cond_label_prefix).str(cond.name).flush()));
             append(ccases, code_case_string(alc, body, gen_cond_enum_elem(buf, opts, cond.name)));
         }
         if (opts->cond_abort) {
