@@ -113,7 +113,7 @@ LOCAL_NODISCARD(Ret fix_mutopt(
         real.var_char = defaults.var_char;
         real.var_cond = defaults.var_cond;
         real.var_cond_table = defaults.var_cond_table;
-        real.var_computed_gotos_table = defaults.var_computed_gotos_table;
+        real.var_cgoto_table = defaults.var_cgoto_table;
         real.var_fill = defaults.var_fill;
         real.var_state = defaults.var_state;
         real.var_nmatch = defaults.var_nmatch;
@@ -196,7 +196,7 @@ LOCAL_NODISCARD(Ret fix_mutopt(
     }
     if (!real.computed_gotos) {
         real.computed_gotos_threshold = defaults.computed_gotos_threshold;
-        real.var_computed_gotos_table = defaults.var_computed_gotos_table;
+        real.var_cgoto_table = defaults.var_cgoto_table;
     }
     if (!real.debug) {
         real.api_debug = defaults.api_debug;
@@ -222,6 +222,9 @@ LOCAL_NODISCARD(Ret fix_mutopt(
     if (real.bitmaps || real.encoding.multibyte_cunit()) {
         real.nested_ifs = true;
     }
+    if (real.computed_gotos_relative) {
+        real.computed_gotos = true;
+    }
     if (real.computed_gotos) {
         real.bitmaps = true;
         real.nested_ifs = true;
@@ -270,7 +273,8 @@ LOCAL_NODISCARD(Ret fix_mutopt(
     if (real.api_style == ApiStyle::FREEFORM && !glob.supported_api_styles_contains("free-form")) {
         RET_FAIL(error("free-form API style is not supported for this backend"));
     }
-    if (!is_default.computed_gotos && !glob.supported_features_contains("computed-gotos")) {
+    if (!(is_default.computed_gotos && is_default.computed_gotos_relative) &&
+        !glob.supported_features_contains("computed-gotos")) {
         RET_FAIL(error("`computed-gotos` feature is not supported for this backend"));
     }
     if (!is_default.bitmaps && !glob.supported_features_contains("bitmaps")) {
@@ -302,7 +306,7 @@ LOCAL_NODISCARD(Ret fix_mutopt(
     if (real.fill_eof != NOEOF) {
         if (real.bitmaps || real.computed_gotos) {
             RET_FAIL(error("`re2c:eof` configuration cannot be used with options --bit-vectors "
-                           "and --computed gotos"));
+                           "and --computed-gotos"));
         }
         if (real.fill_eof >= real.encoding.cunit_count()) {
             RET_FAIL(error("EOF exceeds maximum code unit value for given encoding"));
@@ -847,6 +851,10 @@ static bool eval_cond(
             return opts->monadic;
         case StxGOpt::LOOP_LABEL:
             return !opts->label_loop.empty();
+        case StxGOpt::CGOTO_RELATIVE:
+            return opts->computed_gotos_relative;
+        case StxGOpt::FN_THROW:
+            return !opts->fn_throw.empty();
         }
     }
     UNREACHABLE();
