@@ -75,9 +75,9 @@ void warn_undefined_control_flow(const Skeleton& skel) {
 
         if (i.arc == node.arcs.begin()) {
             // DFS recursive enter
-            if (node.rule != Rule::NONE && node.rule != skel.dfa.eof_rule) {
+            if (node.rule != Rule::NONE) {
                 // accepting path, terminate recursion
-            } else if (node.end() || (use_eof_rule && node.rule == Rule::NONE)) {
+            } else if (node.end() || (use_eof_rule && node.eof_rule == Rule::NONE)) {
                 // found path to default state
                 get_path_on_stack(path, stack, i.node);
                 paths.push_back(path);
